# NOTE: this class has been translated from Java.
# Do not modify this file, changes will be overwritten.

#package orbitgt.mapping3d.sdk.viewer;

from typing import Generic,TypeVar
T = TypeVar('T')

from .....com.orbitgis.lib.lang.Boolean import Boolean
from .....com.orbitgis.lib.lang.Double import Double
from .....com.orbitgis.lib.lang.Integer import Integer

from .....com.orbitgis.lib.util.TreeMap import TreeMap
from .....com.orbitgis.tool.Strings import Strings
from .....orbitgt.mapping3d.sdk.viewer.AList import AList
from .....orbitgt.mapping3d.sdk.viewer.IMap import IMap

#
# Class AMap defines a map of typed key-value pairs.
#

class AMap(Generic[T],IMap):
    # The generic type */
    #private String genericType
    # The elements */
    #private TreeMap[str,T] elements

    #
    # Create a new map.
    # @param genericType the generic type of the elements in the map, like MapObject.TYPE for example.
    #
    def __init__(self,genericType: str):
        self.genericType = genericType
        self.elements = TreeMap[str,T]()

    #
    # Get the generic type of the elements.
    # @return the generic type of the elements, like MapObject.TYPE for example.
    #
    def getGenericType(self) -> str:
        return self.genericType

    #
    # Get a value.
    # @param key the key.
    # @return the value (null if not found).
    #
    def get(self,key: str) -> T:
        return self.elements.get(key)

    #
    # Get a value with an optional default.
    # @param key the key.
    # @param defaultValue the default value.
    # @return the value, or the default value if the key was not found.
    #
    def get2(self,key: str,defaultValue: T) -> T:
        element = self.elements.get(key)
        if (element==None): return defaultValue
        return element

    #
    # Set a value.
    # @param key the key.
    # @param value the value.
    #
    def set(self,key: str,value: T) -> None:
        self.elements.put(key,value)

    #
    # Get an object value.
    # @param key the key.
    # @param defaultValue the default value to return if the key is not found.
    # @return a value.
    #
    def getObject2(self,key: str,defaultValue: object) -> object:
        element = self.elements.get(key)
        if (element==None): return defaultValue
        return element

    #
    # Set an object value.
    # @param key the key.
    # @param value the value to set.
    #
    def setObject(self,key: str,value: object) -> None:
        self.elements.put(key,value)

    #
    # Get a string value.
    # @param key the key.
    # @param defaultValue the default value to return if the key is not found.
    # @return a value.
    #
    def getString2(self,key: str,defaultValue: str) -> str:
        element = self.elements.get(key)
        if (element==None): return defaultValue
        return Strings.toString(element)

    #
    # Set a string value.
    # @param key the key.
    # @param value the value to set.
    #
    def setString(self,key: str,value: str) -> None:
        self.setObject(key,value)

    #
    # Get an integer value.
    # @param key the key.
    # @param defaultValue the default value to return if the key is not found.
    # @return a value.
    #
    def getInt2(self,key: str,defaultValue: int) -> int:
        element = self.elements.get(key)
        if (element==None): return defaultValue
        svalue = Strings.toString(element)
        if (Strings.equals(svalue,"undefined")): return defaultValue
        return Integer.parseInt(svalue)

    #
    # Set an integer value.
    # @param key the key.
    # @param value the value to set.
    #
    def setInt(self,key: str,value: int) -> None:
        self.setObject(key,Integer(value))

    #
    # Get a float value.
    # @param key the key.
    # @param defaultValue the default value to return if the key is not found.
    # @return a value.
    #
    def getFloat2(self,key: str,defaultValue: float) -> float:
        element = self.elements.get(key)
        if (element==None): return defaultValue
        svalue = Strings.toString(element)
        if (Strings.equals(svalue,"undefined")): return defaultValue
        return Double.parseDouble(svalue)

    #
    # Set a float value.
    # @param key the key.
    # @param value the value to set.
    #
    def setFloat(self,key: str,value: float) -> None:
        self.setObject(key,Double(value))

    #
    # Get a boolean value.
    # @param key the key.
    # @param defaultValue the default value to return if the key is not found.
    # @return a value.
    #
    def getBoolean2(self,key: str,defaultValue: bool) -> bool:
        element = self.elements.get(key)
        if (element==None): return defaultValue
        svalue = Strings.toString(element)
        if (Strings.equals(svalue,"undefined")): return defaultValue
        return Boolean.parseBoolean(svalue)

    #
    # Set a boolean value.
    # @param key the key.
    # @param value the value to set.
    #
    def setBoolean(self,key: str,value: bool) -> None:
        if (value): self.setObject(key,Boolean.TRUE)
        else: self.setObject(key,Boolean.FALSE)

    #
    # Remove an element.
    # @param key the key of the element to remove.
    #
    def remove(self,key: str) -> None:
        self.elements.remove(key)

    #
    # Clear the map.
    #
    def clear(self) -> None:
        self.elements.clear()

    #
    # List all keys.
    # @return all keys.
    #
    def listKeys(self) -> AList[str]:
        keys = AList[str]("String")
        for key in self.elements.keySet(): keys.add(key)
        return keys

    #
    # List all values.
    # @return all values.
    #
    def listValues(self) -> AList[T]:
        values = AList[T](self.genericType)
        for value in self.elements.values(): values.add(value)
        return values

    #
    # Convert the object to a string.
    # @return the string form of the object.
    #
    def toString(self) -> str:
        return "[AMap:type="+self.genericType+"]"

    # haxe add begin
    #    #if js
    #    /**
    #     * Returns this instance as a plain javascript object.
    #     * @return an javascript object instance.
    #     */
    #    public function toObject(): Dynamic
    #    {
    #        var instance = {};
    #        for (key in this.listKeys().toArray())
    #        {
    #            Reflect.setField(instance,key,this.get(key));
    #        }
    #        return instance;
    #    }
    #    #end
    # haxe add end

    # typescript add begin
    #    /**
    #     * Returns this instance as a plain javascript object.
    #     * @return an javascript object instance.
    #     */
    #    public toObject(): object
    #    {
    #        let instance: object = {};
    #        for(let key of this.listKeys().toArray())
    #        {
    #            instance[key] = this.get(key);
    #        }
    #        return instance;
    #    }
    # typescript add end
