from qgis.core import QgsFeature,QgsFeatureRenderer,QgsFields,QgsLineSymbol,QgsRenderContext,QgsSymbol
from ..orbitgt.mapping3d.sdk.viewer.AList import AList
from ..orbitgt.mapping3d.sdk.viewer.MapObject import MapObject
from .MapObjectSymbolLayer import MapObjectSymbolLayer

# Class MapObjectRenderer implements a renderer for a list of MapObjects
#
class MapObjectRenderer(QgsFeatureRenderer):
    def __init__(self):
        QgsFeatureRenderer.__init__(self,"MapObjectRenderer")
        self.symbolLayer = MapObjectSymbolLayer()
        self.symbol = QgsLineSymbol.createSimple({})
        self.symbol.deleteSymbolLayer(0)
        self.symbol.appendSymbolLayer(self.symbolLayer)
 
    def clone(self) -> QgsFeatureRenderer:
        other = MapObjectRenderer()
        other.setMapObjectList(self.mapObjects)
        return other
    
    # Call this method to set the list of MapObjects to render
    # Make sure the QgsFeature objects have an integer "id" attribute that defines the index of the corresponding MapObject in the list
    #
    def setMapObjectList(self,mapObjects: AList[MapObject]):
        self.mapObjects = mapObjects
 
    # Called by QGIS when a feature is being rendered
    #
    def symbolForFeature(self,feature: QgsFeature,context: QgsRenderContext) -> QgsSymbol:
        mapObjectId = feature.attribute("id")
        mapObject = None
        if (isinstance(mapObjectId,int)):
            if (mapObjectId>=0) and (mapObjectId<self.mapObjects.size()):
                mapObject = self.mapObjects.get(mapObjectId)
        self.symbolLayer.setMapObject(mapObject,mapObjectId)
        return self.symbol
 
    def startRender(self,context: QgsRenderContext,fields: QgsFields):
        self.symbol.startRender(context)
 
    def stopRender(self,context: QgsRenderContext):
        self.symbol.stopRender(context)
 
    def usedAttributes(self,context: QgsRenderContext):
        return ["id"]
